//
//  block.cpp
//  Lily
//
//  Created by KMY on 2015/05/09.
//  Copyright (c) 2015年 KMY. All rights reserved.
//

#include "block.h"
#include "macro.h"
#include "function.h"
#include "message.h"

Block::Block(Block* parent) {
	parentBlock = parent;
	firstVariable = 0;
	firstMacro = 0;
	firstFunction = 0;
}

Block::~Block() {
	delete firstVariable;
	delete firstMacro;
	delete firstFunction;
}

void Block::addVariable(Variable *obj) {
	if (firstVariable) {
		firstVariable->add(obj);
	} else {
		firstVariable = obj;
	}
}

void Block::addMacro(Macro *obj) {
	if (firstMacro) {
		firstMacro->add(obj);
	} else {
		firstMacro = obj;
	}
}

Block* Block::getParentBlock() {
	return parentBlock;
}

Variable* Block::searchVariable(const lchar *vn) {
	
	// このブロックの中から同名のものを探す
	Variable* p = firstVariable;
	while (p) {
		if (Character::equal(vn, p->getName())) return p;
		p = p->next();
	}
	
	// なければ親ブロックの中から探す
	if (parentBlock) return parentBlock->searchVariable(vn);
	else return 0;
}

Macro* Block::searchMacro(const lchar *vn, Variable* param) {
	
	// このブロックの中から同名のものを探す
	Macro* p = firstMacro;
	while (p) {
		if (p->checkParam(param, vn)) return p;
		p = reinterpret_cast<Macro*>(p->next());
	}
	
	// なければ親ブロックの中から探す
	if (parentBlock) return parentBlock->searchMacro(vn, param);
	else return 0;
}

